/****************************************************************************
**
** Copyright (C) 2006-2009 fullmetalcoder <fullmetalcoder@hotmail.fr>
**
** This file is part of the Edyuk project <http://edyuk.org>
** 
** This file may be used under the terms of the GNU General Public License
** version 3 as published by the Free Software Foundation and appearing in the
** file GPL.txt included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef _QDOCUMENT_LINE_H_
#define _QDOCUMENT_LINE_H_

#include "qce-config.h"

/*!
	\file qdocumentline.h
	\brief Definition of the QDocumentLine class
*/

#include "qformat.h"

class QPoint;
class QString;

class QDocument;
class QDocumentMark;
class QDocumentLineHandle;

class QCE_EXPORT QDocumentLine
{
	friend class QDocumentLineHandle;
	friend class QDocumentCursorHandle;
	
	public:
		enum State
		{
			None				= 0,
			Hidden				= 1,
			CollapsedBlockStart	= 2,
			CollapsedBlockEnd	= 4,
			
			UseLayout			= 16,
			TextChanged			= 32,
			FormattingChanged	= 64,
			HasTabs				= 128
		};
		
		Q_DECLARE_FLAGS(States, State);
		
		QDocumentLine();
		explicit QDocumentLine(const QDocument *doc);
		QDocumentLine(const QDocumentLine& line);
		QDocumentLine(const QDocument *d, QDocumentLineHandle *h = 0);
		
		~QDocumentLine();
		
		bool isNull() const;
		bool isValid() const;
		
		inline bool operator == (const QDocumentLineHandle* h) const
		{
			return m_handle == h;
		}
		
		inline bool operator != (const QDocumentLineHandle* h) const
		{
			return m_handle != h;
		}
		
		bool operator == (const QDocumentLine& l) const;
		bool operator != (const QDocumentLine& l) const;
		
		QDocumentLine& operator = (const QDocumentLine& l);
		
		QString text() const;
		
		int lineNumber() const;
		
		int length() const;
		int lineSpan() const;
		
		int firstChar() const;
		int lastChar() const;
		
		int indent() const;
		
		int nextNonSpaceChar(int pos) const;
		int previousNonSpaceChar(int pos) const;
		
		inline QString indentation() const
		{ int idx = firstChar(); return idx != -1 ? text().left(idx) : text(); }
		
		inline bool isHidden() const
		{ return hasFlag(Hidden); }
		
		bool hasFlag(State s) const;
		bool hasAnyFlag(int s) const;
		void setFlag(State s, bool y = true);
		
		int wrappedLineForCursor(int cpos) const;
		
		int documentOffsetToCursor(int x, int y) const;
		void cursorToDocumentOffset(int cpos, int& x, int& y) const;
		
		QPoint cursorToDocumentOffset(int cpos) const;
		
		// TODO : move away...
		void addMark(int id);
		void removeMark(int id);
		void toggleMark(int id);
		
		QList<int> marks() const;
		bool hasMark(int id) const;
		
		bool hasOverlay(int fid) const;
		QList<QFormatRange> overlays() const;
		
		void clearOverlays();
		void addOverlay(const QFormatRange& over);
		void removeOverlay(const QFormatRange& over);
		
		const QVector<int>& tokens() const;
		
		void clearTokens();
		void addToken(int index, int length, int tokenId);
		void setTokens(const QVector<int>& tokens);
		
		const QVector<int>& contexts() const;
		
		void clearContexts();
		void addContext(int index, int length, int contextId);
		void setContexts(const QVector<int>& contexts);
		
		inline const QDocument* document() const
		{ return m_doc; }
		
		inline QDocumentLineHandle* handle() const
		{ return m_handle; }
		
	private:
		const QDocument *m_doc;
		QDocumentLineHandle *m_handle;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(QDocumentLine::States)

#endif
