/****************************************************************************
**
** Copyright (C) 2006-2009 fullmetalcoder <fullmetalcoder@hotmail.fr>
**
** This file is part of the Edyuk project <http://edyuk.org>
** 
** This file may be used under the terms of the GNU General Public License
** version 3 as published by the Free Software Foundation and appearing in the
** file GPL.txt included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef _QLANGUAGE_DEFINITION_H_
#define _QLANGUAGE_DEFINITION_H_

#include "qce-config.h"

/*!
	\file qlanguagedefinition.h
	\brief Definition of the QLanguageDefinition class.
	
	\see QLanguageDefinition
*/

#include "qformat.h"
#include "qdocumentbuffer.h"

#include <QStringList>

class QKeyEvent;
class QDocument;
class QDocumentCursor;

#define QCE_FOLD_FLAGS(flags, open, close) ((flags) | (open & QLanguageDefinition::OpenMask) | ((close << 12) & QLanguageDefinition::CloseMask))
#define QCE_FOLD_OPEN_COUNT(flags) ((flags) & QLanguageDefinition::OpenMask)
#define QCE_FOLD_CLOSE_COUNT(flags) (((flags) & QLanguageDefinition::CloseMask) >> 12)

class QCE_EXPORT QLanguageDefinition
{
	public:
		enum ContextIdPacking
		{
			ContextIdMask	= 0x00ffffff,
			ReservedMask	= 0xff000000,
			
			Language		= 0x80000000
		};
		
		enum TokenIdPacking
		{
			TokenIdMask	= 0x00ffffff,
			FlagsMask	= 0xff000000,
			
			Paren		= 0x01000000,
			OpenParen	= 0x02000000,
			CloseParen	= 0x04000000,
			
			AutoIndent	= 0x08000000,
			BlockFold	= 0x10000000,
			BraceMatch	= 0x20000000
		};
		
		/// Collapse state of a line
		enum CollapseFlag
		{
			None		= 0x00000000,		///< The line cannot be collapsed nor expanded
			Collapsible	= 0x10000000,		///< The line is expanded and can thus be collapsed
			Collapsed	= 0x20000000,		///< The line is collapsed and can thus be expanded
			Closure		= 0x40000000,		///< The line is expanded and mark the end of a block
			
			CloseMask	= 0x00fff000,		///< Number of actual closing fold mark
			OpenMask	= 0x00000fff		///< Number of actual open fold mark
		};
		
		Q_DECLARE_FLAGS(CollapseState, CollapseFlag);
		
		QLanguageDefinition();
		virtual ~QLanguageDefinition();
		
		int languageId() const;
		
		virtual QString language() const = 0;
		virtual QStringList extensions() const = 0;
		
		virtual int tokenize(QDocument *d, int line, int count);
		virtual int tokenize(QDocumentBuffer::iterator& it, int& line, int& column, int extent, int depth, int cxt);
		
		virtual QString singleLineComment() const;
		
		virtual QString defaultLineMark() const;
		
		virtual void match(QDocumentCursor& c);
		virtual void clearMatches(QDocument *d);
		
		virtual QString indent(const QDocumentCursor& c);
		virtual bool unindent (const QDocumentCursor& c, const QString& ktxt);
		
		virtual void expand(QDocument *d, int line);
		virtual void collapse(QDocument *d, int line);
		virtual int blockFlags(QDocument *d, int line, int depth = 0) const;
		
		virtual bool parenMatch(int tokenId1, const QStringRef prod1, int tokenId2, const QStringRef prod2) const;
		
		static int id(const QString& lang);
		static QLanguageDefinition* instance(int id);
		
		QString contextName(int id) const;
		int contextId(const QString& s) const;
		
		virtual int tokenFormat(int tokenId) const;
		virtual int contextFormat(int contextId) const;
		
	protected:
		int addContext(const QString& cxt);
		
	private:
		int m_id;
		
		QList<int> m_contextIds;
		QStringList m_contextNames;
		
		static int m_contextCount;
		static QVector<QLanguageDefinition*> m_definitions;
};

#endif // _QLANGUAGE_DEFINITION_H_
